﻿using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Encodings.SymbolicExpressionTreeEncoding;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Problems.DataAnalysis;

namespace HeuristicLab.Problems.GaussianProcessTuning {
  [StorableClass]
  [Item("GaussianProcessRegressionModel", "A Gaussian process model")]
  public class GaussianProcessRegressionModel : NamedItem, IRegressionModel {
    [Storable]
    private ISymbolicExpressionTree configuration;
    public ISymbolicExpressionTree Configuration { get { return configuration; } }

    [Storable]
    private IRegressionProblemData trainingData;
    [Storable]
    private Interpreter interpreter;

    [StorableConstructor]
    protected GaussianProcessRegressionModel(bool deserializing) : base(deserializing) { }
    protected GaussianProcessRegressionModel(GaussianProcessRegressionModel original, Cloner cloner)
      : base(original, cloner) {
      this.configuration = cloner.Clone(original.configuration);
      this.trainingData = cloner.Clone(original.trainingData);
      this.interpreter = cloner.Clone(original.interpreter);
    }
    public GaussianProcessRegressionModel(ISymbolicExpressionTree configuration, IRegressionProblemData trainingData, Interpreter interpreter) {
      this.Name = "GaussianProcessRegressionModel";
      this.Description = "A Gaussian process model";
      this.configuration = configuration;
      this.trainingData = trainingData;
      this.interpreter = interpreter;
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new GaussianProcessRegressionModel(this, cloner);
    }

    public IRegressionSolution CreateRegressionSolution(IRegressionProblemData problemData) {
      return new Solution(this, problemData);
    }

    public IEnumerable<double> GetEstimatedValues(Dataset dataset, IEnumerable<int> rows) {
      //if (!rows.Any()) return Enumerable.Empty<double>();

      //double[] means, variances;
      //interpreter.EvaluateGaussianProcessConfiguration(configuration, trainingData, dataset, rows, out means,
      //                                                 out variances);
      //return means;
      return rows.Select(x => 0.0);
    }
  }
}
